import re
try:
    from lgrlist import LGRLIST
except ImportError:
    from langsci.lgrlist import LGRLIST

NON_CCBY_LIST = [148, 149, 234]

PRESOURCELINE = (
    r"(?P<presourceline>\\(ili|langinfo){(?P<language_name>.*?)?}.*\\\\ *\n)?"
)
SOURCELINE = r"\\gll[ \t]*(?P<sourceline>.*?) *?\\\\\n"
IMTLINE1 = r"[ \t]*(?P<imtline1>.*?) *?\\\\\n+"  # some authors add multiple newlines before the translation
IMTLINE2 = r"([ \t]*(?P<imtline2>.*?) *?\\\\\n+)?"  # some authors add multiple newlines before the translation
TRSLINE = r"[ \t]*\\(glt|trans)[ \t\n]*(?P<translationline>.*?)\n"
LGRPATTERN_UPPER = re.compile(r"\\(%s)({})?" % "|".join(LGRLIST))
LGRPATTERN_LOWER = re.compile(r"(%s)({})?" % "|".join([s.lower() for s in LGRLIST]))
LGRPATTERN_UPPER_LOWER = re.compile(
    r"(%s)({})?" % "|".join([s[0] + s[1:].lower() for s in LGRLIST])
)
#braces after a space or after the beginning of the line
BRACESPATTERN = re.compile(r"(?<=.^| ){([^}{ ]+)}")
GLL = re.compile(PRESOURCELINE + SOURCELINE + IMTLINE1 + IMTLINE2 + TRSLINE)
TEXTEXT = re.compile(r"\\text(.*?)\{(.*?)\}")
TEXSTYLEENVIRONMENT = re.compile(r"\\\\textstyle[A-Z][A-Za-z].*?{(.*?)}")
INDEXCOMMANDS = re.compile(r"\\\\i[sl]{(.*?)}")
LABELCOMMANDS = re.compile(r"\\\\label{(.*?)}")
CITATION = re.compile(r"\\cite[altpv]*(\[.*?\])?\{(.*?)\}")

STARTINGQUOTE = "`‘"
ENDINGQUOTE = "'’"
TEXREPLACEMENTS = [
    (r"\_", "_"),
    (r"\texttimes", "×"),
    (r"\textquotesingle", "'"),
    (r"\textquotedbl", '"'),
    (r"\textprimstress", "ˈ"),
    (r"\textbackslash", r"\\"),
    (r"\textbar", "|"),
    (r"\textasciitilde", "~"),
    (r"\textless", "<"),
    (r"\textgreater", ">"),
    (r"\textrightarrow", "→"),
    (r"\textalpha", "α"),
    (r"\textbeta", "β"),
    (r"\textgamma", "γ"),
    (r"\textdelta", "δ"),
    (r"\textepsilon", "ε"),
    (r"\textphi", "φ"),
    (r"\textupsilon", "υ"),
    (r"\newline", " "),
    (r"{\ꞌ}", "ꞌ"),
    (r"{\ob}", "["),
    (r"{\cb}", "]"),
    (r"{\db}", " "),
    (r"\nobreakdash", ""),
    (r"\textendash", "-"),
    (r"\footnotesize", "-"),
    (r"\footnotemark", "-"),
    (r"\langle", "<"),
    (r"\rangle", ">"),
    (r"\ldots", "..."),
    (r"\dots", "..."),
    (r"\MakeLowercase", ""),
    (r"\nopagebreak", ""),
    (r"\pagebreak", ""),
    (r"\sout", r"\textstrikeout"),
    (r"\uline", r"\textunderline"),
    (r"\underline", r"\textunderline"),
    (r"\emph", r"\textemph"),
    (r"\textup", ""),
    (r"\url", ""),
    (r"\chapref", "Chapter "),
    (r"\sectref", "Section "),
    (r"\figure", "Figure "),
    (r"\tabref", "Table "),
    (r"\appref", "Appendix "),
    (r"\fnref", "Footnote "),
    (r"\ref", ""),
    (r"\forall", "∀"),
    (r"\exists", "∃"),
    (r"\xspace", ""),
    (r"\bluebold", r"\textbf"),
    (r"\blueboldSmallCaps", r"\textsc"),
    (r"\tsc", r"\textsc"),
    (r"\ili", ""),
    (r"\isi", ""),
    (r"\sc", r"\scshape"),
    (r"\it ", r"\itshape "),
    (r"\bf ", r"\bfseries "),
    (r"\lq", '"'),
    (r"\{", "{"),
    (r"\}", "}"),
    # (r"\label", ''),
    (r"\hfill", " "),
    (r"\vfill", " "),
    (r"\sqt", " "),
    (r"\fbox", " "),
    (r"\spacebr", ""),
    (r"\ule", ""),
    (r"\ulp", ""),
    (r"\oldstylenums", ""),
    (r"\normalfont", ""),
    (r"\NORMALFONT", ""),
    (r"\longrightarrow", "→"),
    (r"\leftrightarrow", "⇔"),
    (r"\emptyset", "ø"),
    (r"\varnothing", "ø"),
    (r"\USEmptySet", "ø"),
    (r"\Tilde", "~"),
    (r"\NORMALFONT", ""),
    (r"\relax", ""),
    (r"\raisebox", ""),
    (r"\quotetrans", ""),
    (r"\qquad", ""),
    (r"\quad", ""),
    (r"\nobreakdash", ""),
    (r"\NOBREAKDASH", ""),
    (r"\newpage", ""),
    (r"\mytrans", r"\glt"),
    (r"\minsp", ""),
    (r"\makebox", ""),
    (r"\longrule", "__"),
    (r"\MakeUpperCase", ""),
    (r"\MakeUppercase", ""),
    (r"\linebreak", ""),
    (r"\linewidth", ""),
    (r"\interfootnotelinepenalty", ""),
    (r"\hskip", ""),
    (r"\deff", "\def"),
    (r"\smash", ""),
    (r"\enquote", ""),
    (r"\Emph", ""),
    (r"\sim", "~"),
    (r"\USgreater", ">"),
    (r"\USsmaller", "<"),
    (r"\USQMark", "?"),
    (r"\slash", "/"),
    (r"\OLDSTYLENUMS", "/"),
    (r"\thirdperson", "3"),
    (r"\textglotstop", "ʔ"),
    (r"\beltl", "ł"),
    (r"\textltailn", "ɲ"),
    (r"\textopeno", "ɔ"),
    (r"\textstrikeout", ""),
    (r"\textunderscore", ""),
    (r"\textstyleEmphasizedVernacularWords", r"\emph"),
    (r"\protect", ""),
    (r"\\", " "),
    (r"\citealt", ""),
    (r"\Highlight", r"\textbf"),
    (r"\sg", r"SG"),
    (r"\rouge", r"\textred"),
    (r"\bleu", r"\textblue"),
    (r"\mbox", r""),
    (r"\ddagger", r"‡"),
    (r"\dagger", r"†"),
    (r"\downstep", r"ꜜ"),
    (r"\emph", r"\textemph"),
    (r"\kern2pt", r""),
    (r"\,", r" "),
    (r"\Rightarrow", r"→"),
    (r"\rm", r""),
    (r"\#", r"#"),
    (r"\Third", r"3"),
    (r"\super", r"\textsuperscript"),
    (r"\expo", r"\textsuperscript"),
    (r"\small", r"\textsmall"),
    (r"\footnotesize", r"\textsmall"),
    (r"\scriptsize", r"\textsmall"),
    (r"\tiny", r"\textsmall"),
    (r"\USGreater", r">"),
    (r"\USSmaller", r"<"),
    (r"\exi", r" "),
    (r"\itshape", r"\textit"),
    (r"\upshape", r""),
    (r"\@", r"@"),
    (r"\"=", r"-"),
    (r"{\scshape", r"\textsc{"),
    (r"\gsc", r"\textsc"),
    (r"\glemph", r"\textemph"),
    (r"\O", "ø"),
    (r"{\R}", "REALIS"),
    (r"\circ", "°"),
    (r"\stem", ""),
    (r"\z", ""),
    (r"\ex ", " "),
    (r"\ix{", r"\textsubscript{"),
    (r"\textsmallskip", r""),
    (r"\smallskip", r""),
    (r"\medskip", r""),
    (r"\bigskip", r""),
    (r"\extrans", r"\glt "),
    (r"\glft", r"\glt "),
    (r"\trad", r"\glt "),
    (r"\op", r"("),
    (r"\cp", r")"),
    (r"\ob", r"["),
    (r"\cb", r"]"),
    (r"\db", r" "),
    (r"\llap", r""),
    (r"\textemdash", r"—"),
    (r"\Corpus", r""),
    (r"\flobv{}", r"→3'"),
    (r"\mc{", r"\textsc{"),
    (r"\tkal", r"XaYaZ"),
    (r"\tpie", r"XiY̯eZ"),
    (r"\tpua", r"XuY̯aZ"),
    (r"\mpua", r"meXuY̯aZ"),
    (r"\thif", r"heXYiZ"),
    (r"\thuf", r"huXYaZ"),
    (r"\mhuf", r"muXYaZ"),
    (r"\thit", r"hitXaY̯eZ"),
    (r"\tnif", r"niXYaZ"),
    (r"\gloss{", r"\textsc{"),
    (r"{\LINK}", r"LINK"),
    (r"\mas{}", r"M"),
    (r"\ipa{}", r""),
    (r"\tss", r"\textsubscript"),
    (r"\tsp", r"\textsuperscript"),
    (r"\Tsg{}", r"3.SG"),
    (r"\Tpl{}", r"3.PL"),
    (r"\oneS", r"1.SG"),
    (r"\twoS", r"2.SG"),
    (r"\prs{}", r"PRS"),
    (r"\tld", r"~"),
    (r"\itshape", r"\textit{"),
    (r"\scshape", r"\textsc{"),
    (r"\bf", r"\bfseries"),
    (r"\bfseries", r"\textbf{"),
    (r"\nom", r"NOM"),
    (r"\acc", r"ACC"),
    (r"{\pl}", r"PL"),
    (r"{\pst}", r"PST"),
    (r"{\prs}", r"PRS"),
    (r"\lptcp", r"l-PTCP"),
    # greekletters
]

# remove these together with their argument
TEXTARGYANKS = [
    "ConnectHead",
    "ConnectTail",
    "hphantom",
    "japhdoi",
    "phantom",
    "vspace",
    "vspace\*",
    "begin",
    "end",
    "is",
    "il",
    "hspaceThis",
    "hspace",
    "hspace\*",
    "ilt",
    "ist",
]

ONE_LANGUAGE_BOOKS = {
#   ID  ISO6393 Glottocode   Name
    17: ("sje", "pite1240", "Pite Saami"),
    66: ("ybh", "yakk1236", "Yakkha"),
    67: ("mhl", "mauw1238", "Mauwake"),
    78: ("pmy", "papu1250", "Papuan Malay"),
    82: ("phl", "phal1254", "Palula"),
    85: ("fpe", "fern1234", "Pichi"),
    118: ("mlw", "molo1266", "Moloko"),
    124: ("rap", "rapa1244", "Rapa Nui"),
    212: ("tci", "wara1294", "Komnzo"),
    250: ("dar", "sanz1248", "Sanzhi Dargwa"),  #      (Iso is not precise here)
    298: ("gyi", "gyel1242", "Gyeli"),
    295: ("jya", "japh1234", "Japhug"),  # (Iso is not precise here)
    308: ("roh", "surs1245", "Tuatschin"),  # (Iso is not precise here)
    245: ("dga", "sout2789", "Dagaare"),
    98: ("ikx", "ikkk1242", "Ik"),
    326: ("ruc", "ruul1235", "Ruruuli"),
    287: ("nyy", "nyak1261", "Nyakyusa"),
    109: ("nru", "yong1288", "Yongning Na"),
    225: ("dar", "mege1234", "Mehweb"),
    228: ("aaz", "amar1273", "Amarasi"),
    150: ("rus", "russ1263", "Russian"),
    329: ("swe", "swed1254", "Swedish"),
    16: ("ita", "ital1282", "Italian"),
}



SUPERSEDED = [22, 25, 46, 141, 144, 149, 195]
FRENCH = [27, 143]
GERMAN = [101, 234, 155, 134, 116]
PORTUGUESE = [160, 200]
CHINESE = [177]
SPANISH = [236]
